/***************************************************************************
                          kdbx.cpp  -  description
                             -------------------
    begin                : Sat May 12 19:36:13 BST 2001
    copyright            : (C) 2001 by 
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>

// application specific includes
#include "kdbx.h"
#include "kdbxdoc.h"
#include "kdbxview.h"
#include "kdbxfldexportdlg.h"

#define ID_STATUS_MSG 1

KdbxApp::KdbxApp(QWidget* , const char* name):KMainWindow(0, name)
{
  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initDocument();
  initView();
	
  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
//  fileClose->setEnabled(false);
}

KdbxApp::~KdbxApp()
{

}

void KdbxApp::initActions()
{
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
	fileOpenRecentDir = new KRecentFilesAction(i18n("Open Recent Dir"), 0, this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection(), "file_recent_dir");
//  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
	viewPreviewPane = new KToggleAction(i18n("&Preview Pane"), 0, this, SLOT(slotShowPane()), actionCollection(), "options_show_preview");
	
	exportDir = new KAction(i18n("&Folder..."), 0, this, SLOT(slotExportDir()), actionCollection(), "export_dir");
	exportSelEmail = new KAction(i18n("&Selected..."), 0, this, SLOT(slotExportSelEmail()), actionCollection(), "export_sel_emails");
	
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileOpenRecentDir->setStatusText(i18n("Opens a recently used directory"));
//  fileClose->setStatusText(i18n("Closes the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
	viewPreviewPane->setStatusText(i18n("Enables/disables the preview box"));
	
	exportDir->setStatusText(i18n("Export the selected folder"));
	exportSelEmail->setStatusText(i18n("Export the selected emails"));

//	toolBar("mainToolBar")->insertButton(BarIcon("redo.png"), 500, true, "Export Dir");
  // use the absolute path to your kdbxui.rc file for testing purpose in createGUI();
  createGUI();
}


void KdbxApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
  progress = new KProgress(statusBar());
  progress->hide();
}

void KdbxApp::initDocument()
{
  doc = new KdbxDoc(this);
  connect(doc, SIGNAL(signalStatusBar(const QString&)), this, SLOT(slotStatusMsg(const QString &)));
  connect(this, SIGNAL(signalShowPreview(int)), doc, SLOT(slotShowPreview(int)));
  connect(doc, SIGNAL(signalUpdateProgress(int)), this, SLOT(slotUpdateProgress(int)));
}

void KdbxApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KdbxView(this);
  connect(view, SIGNAL(signalStatusBar(const QString &)), this, SLOT(slotStatusMsg(const QString &)));

  doc->addView(view);
  setCentralWidget(view);	
  setCaption("");
}

void KdbxApp::openDocumentFile(const KURL& url)
{
	if (!url.isEmpty()) {
    slotStatusMsg(i18n("Opening file..."));
    doc->openDocument(url);
    fileOpenRecent->addURL( url );
    slotStatusMsg(i18n("Ready."));
  }
}


KdbxDoc *KdbxApp::getDocument() const
{
  return doc;
}

void KdbxApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("Show Preview",viewPreviewPane->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  config->writeEntry("Export Recurse", exportRecurse);
  config->writeEntry("Export Subdir", exportSubdir);
  fileOpenRecent->saveEntries(config,"Recent Files");
  fileOpenRecentDir->saveEntries(config, "Recent Dir");
  view->saveOptions(config);
}


void KdbxApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();

	bool bViewPreviewPane = config->readBoolEntry("Show Preview", true);
	viewPreviewPane->setChecked(bViewPreviewPane);
	slotShowPane();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
  exportRecurse = config->readBoolEntry("Export Recurse");
	exportSubdir = config->readBoolEntry("Export Subdir");
	
  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");
	fileOpenRecentDir->loadEntries(config, "Recent Dir");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
  view->loadOptions(config);
}

void KdbxApp::saveProperties(KConfig *_cfg)
{
  KURL url=doc->URL();	
  _cfg->writeEntry("filename", url.url());
  QString tempname = kapp->tempSaveName(url.url());
  QString tempurl= KURL::encode_string(tempname);
}


void KdbxApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  if(!filename.isEmpty())
  {
    doc->openDocument(url);
    setCaption(url.fileName(),false);
  }
}		

bool KdbxApp::queryClose()
{
  return true;
}

bool KdbxApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KdbxApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  KURL url=KFileDialog::getOpenURL(":<a>",
        i18n("*.dbx|Outlook 5.x Files\n*|All files"), this, i18n("Open..."));
  if(!url.isEmpty())
  {
  	qDebug(url.filename());
  	if (qstricmp(url.filename(), "Folders.dbx")==0) { //true if they are the same
  		qDebug("Adding to RecentDir");
  		fileOpenRecentDir->addURL(url);
  	} else {
  		qDebug("Adding to Recent");
  		fileOpenRecent->addURL(url);
  	}
    doc->openDocument(url);
  }
  slotStatusMsg(i18n("Ready."));
}

void KdbxApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	
  doc->openDocument(url);
  setCaption(url.fileName(), false);

  slotStatusMsg(i18n("Ready."));
}

void KdbxApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();

  slotStatusMsg(i18n("Ready."));
}

void KdbxApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
				break;
    }
  }	
  slotStatusMsg(i18n("Ready."));
}

void KdbxApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KdbxApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void KdbxApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
//  printf("Changing Status bar to %s\n", text.latin1());
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

/** Enable/Disable the preview pane */
void KdbxApp::slotShowPane(){
	if (!viewPreviewPane->isChecked()) {
		emit signalShowPreview(0);
	} else {
		emit signalShowPreview(1);
	}
	slotStatusMsg(i18n("Ready."));
}

/** Export the selected folder */
void KdbxApp::slotExportDir(){
	slotStatusMsg(i18n("Exporting Folders..."));
	KdbxFldExportDlg expDir(":<a>", "", this, 0, true);
	expDir.setMode(KFile::Directory);
	expDir.recurse->setChecked(exportRecurse);
	expDir.createSub->setChecked(exportSubdir);
	
	if (expDir.exec()==1) {
  	KURL url = expDir.baseURL();
  	
  	exportRecurse = expDir.recurse->isChecked();
  	exportSubdir = expDir.createSub->isChecked();
  	
  	if (!url.isEmpty()) {
  		view->exportFolder(&url, (exportRecurse?1:0), (exportSubdir?1:0));
	  	KMessageBox::information(this, "Finished Export");
  	}
	}	
	slotStatusMsg(i18n("Ready."));
}

/** Export the selected emails */
void KdbxApp::slotExportSelEmail(){
	slotStatusMsg(i18n("Exporting Emails..."));
  KURL url=KFileDialog::getSaveURL(":<a>",
        i18n("*|All files"), this, i18n("Save..."));

  if (!url.isEmpty()) {
		view->exportSelEmails(&url);
		KMessageBox::information(this, "Finished Export");
	}
	slotStatusMsg(i18n("Ready."));
}

/** Increase the Progess bar by adv */
void KdbxApp::slotUpdateProgress(int adv){
	if (!progress->isVisible() && adv != 100) {
		statusBar()->addWidget(progress, 0, this);
		progress->show();
	}
	progress->setValue(adv);
	if (adv == 100 && progress->isVisible()) {
		progress->hide();
		statusBar()->removeWidget(progress);
	}
}
