/***************************************************************************
                          kdbxview.cpp  -  description
                             -------------------
    begin                : Sat May 12 19:36:13 BST 2001
    copyright            : (C) 2001 by
    email                :
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qprinter.h>
#include <qpainter.h>
#include <qlayout.h>
#include <qlistview.h>

//KDE
#include <klistview.h>
#include <ktextbrowser.h>
#include <kpopupmenu.h>

// application specific includes
#include "libdbx++.h"
#include "kdbxdoc.h"
#include "kdbxview.h"
#include "kdbx.h"
#include "timeconv.h"
#include "kdbxlistitem.h"

//#include <stdlib.h>

KdbxView::KdbxView(QWidget *parent, const char *name) : QWidget(parent, name)
{
  QGridLayout *toplayout = new QGridLayout(this,2,1);
	split1 = new QSplitter(this);
	
 	folderlist = new KListView(split1);
 	folderlist->addColumn("Folder");
// 	folderlist->setColumnWidthMode(0, folderlist->Manual);
 	
// 	folderlist->addColumn("Filename");
	
	split2 = new QSplitter(QSplitter::Vertical, split1);	
 	
 	emaillist = new KListView(split2);
 	emaillist->addColumn("Sender");
 	emaillist->addColumn("Subject");
 	emaillist->addColumn("Arrival Date");
 	 	
 	emaillist->setColumnWidthMode(0, emaillist->Manual);
 	emaillist->setColumnWidthMode(1, emaillist->Manual);
 	emaillist->setColumnWidthMode(2, emaillist->Manual);

 	emaillist->setSorting(2); //date column
 	emaillist->setShowSortIndicator(TRUE);
 	emaillist->setSelectionModeExt(KListView::Konqueror);
 	emaillist->setAllColumnsShowFocus(TRUE);
 	
 	browser = new KTextBrowser(split2);
 	browser->setTextFormat(Qt::PlainText);
 	
	toplayout->addWidget(split1,0,0);
 	toplayout->activate();

 	exportMenu = new KPopupMenu(this, "export");
 	
 	connect (folderlist, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(slotItemChanged(QListViewItem*)));
 	connect (emaillist, SIGNAL(clicked(QListViewItem*/*,const QPoint &, int*/)), this, SLOT(slotEmailClicked(QListViewItem*)));
 	connect (emaillist, SIGNAL(doubleClicked(QListViewItem*,const QPoint &, int)), this, SLOT(slotEmailDblClicked(QListViewItem*)));
}

KdbxView::~KdbxView()
{
}

void KdbxView::connections(KdbxDoc *doc) {
	connect(doc, SIGNAL(signalEmailsLoaded()), this, SLOT(slotEmailsLoaded()));
	connect(this, SIGNAL(signalFolderChanged(const QString&)), doc, SLOT(slotFolderChanged(const QString&)));	
	connect(doc, SIGNAL(signalUpdateViews()), this, SLOT(slotFoldersLoaded()));	
	connect(doc, SIGNAL(signalShowPreview(int)), this, SLOT(slotShowPreview(int)));
}

KdbxDoc *KdbxView::getDocument() const
{
  KdbxApp *theApp=(KdbxApp *) parentWidget();

  return theApp->getDocument();
}

void KdbxView::slotItemChanged(QListViewItem *item) {
	KdbxFolderListItem *folder = (KdbxFolderListItem*) item;
	
	if (folder->data != NULL) {
		emit signalFolderChanged(QString(folder->data->fname()));
	} else {
		emit signalFolderChanged(QString::null);
	}
}

void KdbxView::slotEmailClicked(QListViewItem *item) {
	KdbxListItem *k = (KdbxListItem*)item;
	//show in another pane
	if (!browser->isVisible() || item == NULL) //maybe no item was clicked on
		return;
	emit signalStatusBar("Loading Email...");
	

	if (k->data == NULL || k->data->body() == NULL) {		
		getDocument()->loadEmailBody(k->data);
		if (k->data->body() == NULL)
			browser->setText("ERROR: no data associated with current item");
		else
			browser->setText(k->data->body());
	} else {
		browser->setText(k->data->body());
	}	
	if (k->data->body() != NULL) {
		getDocument()->clearEmailBody(k->data);
	}
	emit signalStatusBar("Ready.");
}

void KdbxView::slotEmailDblClicked(QListViewItem *item) {
	//do something with KTextBrowser here. Dunnowhat tho...
	//might be nice to open a new window containing the contents of the email
	item=item;
}

void KdbxView::slotEmailsLoaded() {
	CEmail *email;
	KdbxDoc *doc = getDocument();
	KdbxListItem *item;
	time_t date;
	char *cdate;
	QFont cur;
	
	emaillist->clear();
	email = doc->emails.first();
	while (email != NULL) {
		date = FileTimeToUnixTime(email->date(), NULL);		
		cdate = ctime(&date);
		cdate[strlen(cdate)-1]='\0'; //remove the CR from the cdate
		item = new KdbxListItem(emaillist, email->sender_name(), email->subject(), cdate);
		item->data = email;		
		email = doc->emails.next();
	}
	browser->setText(NULL);
}	

/** Document has loaded some folders */
void KdbxView::slotFoldersLoaded(){
	CFolder* low=NULL, *cur;
	folderlist->clear();
	
	KdbxDoc *doc = getDocument();
	cur = doc->folderTree.first();
	while (cur != NULL) {
		if (low == NULL || cur->id() < low->id())
			low = cur;
		cur = doc->folderTree.next();
	}
	
	buildTree(low, NULL);
}

void KdbxView::buildTree(CFolder* cur, KdbxFolderListItem *par) {
	KdbxFolderListItem *item;
	
	while (cur != NULL) {
		if (cur->structOK() && cur->name() != NULL) {
			if (par == NULL) {			
				item = new KdbxFolderListItem(folderlist, cur->name());
				item->data = cur;
				item->setOpen(true);
			} else {
				item = new KdbxFolderListItem(par, cur->name());
				item->data = cur;
			}
		}	else {
			if (par == NULL) {
				item = new KdbxFolderListItem(folderlist, "Untitled");
				item->data = NULL;
				item->setOpen(true);
			} else {
				item = new KdbxFolderListItem(par, "Untitled");
				item->data = NULL;
			}
		}
			
		if (cur->child != NULL) {
			buildTree(cur->child, item);
		}
		cur = cur->next;
	}
}

/** Show/hide preview pane */
void KdbxView::slotShowPreview(int i){
	if (i) {
		browser->show();
	} else {
		browser->setText(NULL);
		browser->hide();
	}
}

/** Exports currently selected emails */
void KdbxView::exportSelEmails(KURL* url){
	QList <QListViewItem> list;
	KdbxListItem *item;
	KdbxDoc *doc = getDocument();
	QList <CEmail> emails;
	
	list = emaillist->selectedItems();
	item = (KdbxListItem*) list.first();
	while (item != NULL) {
		emails.append(item->data);
		item = (KdbxListItem*) list.next();
	}
	doc->exportEmails(&emails, url);
	emaillist->selectAll(false);
}

/** Find currently selected folder and pass back to document object */
void KdbxView::exportFolder(KURL *url, int subfold, int subdir){
	KdbxDoc *doc = getDocument();
	CFolder *treeitem;
	KdbxFolderListItem *folder;
	
	if ((folder = (KdbxFolderListItem*) folderlist->selectedItem()) == NULL)
		return;
	
	treeitem = folder->data;
	if (folder != 0) {
		getDocument()->exportFolder(treeitem, url, subfold, subdir, 0);
	}	
}

/** Saves the properties relevant to the view */
int KdbxView::saveOptions(KConfig *cfg){
	cfg->setGroup("view options");
	QValueList <int> x;
	
	int z = emaillist->columns();
	for (int y = 0; y < z; y++) {
		x.append(emaillist->columnWidth(y));
	}
	cfg->writeEntry("Column Width", x);
		
  cfg->writeEntry("Preview Height", split2->sizes()[1]);
  return 1;
}

/** Reload the settings from a config file */
int KdbxView::loadOptions(KConfig *cfg){
	cfg->setGroup("view options");
	
	QValueList <int> l;
	l = split2->sizes();
	l[1] = cfg->readNumEntry("Preview Height", 0);
	split2->setSizes(l);
	
	QValueList <int> x;
	x = cfg->readIntListEntry("Column Width");
	int z = x.count();
	for (int y = 0; y < z; y++) {	
		emaillist->setColumnWidth(y, x[y]);
	}
	return 1;
}
